require 'dotenv'
require 'httparty'
require 'rails'

Dotenv.load

require "inventos_streak/version"
require "inventos_streak/configuration"
require "inventos_streak/base"
require "inventos_streak/pipeline"
require "inventos_streak/box"
require "inventos_streak/stage"
require "inventos_streak/field"

module InventosStreak
  class << self
    attr_writer :configuration

    [:get, :delete].each do |attr|
      define_method attr do |path|
        url = "#{base_url}/#{path}"
        puts "#{attr} #{url}"
        HTTParty.send attr, url, basic_auth: configuration.basic_auth, verify: false
      end
    end

    [:post, :put].each do |attr|
      define_method attr do |path, body|
        url = "#{base_url}/#{path}"
        puts "#{attr} #{url}"
        HTTParty.send attr, url, basic_auth: configuration.basic_auth, body: body
      end

      define_method "#{attr}_json" do |path, body|
        url = "#{base_url}/#{path}"
        puts "#{attr} #{url}"
        HTTParty.send attr, "#{base_url}/#{path}", basic_auth: configuration.basic_auth, body: body.to_json, headers: { 'Content-Type' => 'application/json' }
      end
    end
  end

  def self.configuration
    @configuration ||= Configuration.new
  end

  def self.reset
    @configuration = Configuration.new
  end

  def self.configure
    yield(configuration)
  end

  def self.base_url
    configuration.streak_base_url
  end
end