class InventosStreak::Base
  class << self
    def all
      InventosStreak.get(path)
    end

    def find key, options = {}
      options.merge!(key: key)
      m = self.new options
      InventosStreak.get(m.final_path).map do |k, v|
        m.instance_variable_set("@#{k}", v)
      end
      m
    end

    def belongs_to relation
      parents.push relation
    end

    def has_many relation
      children.push relation
    end

    def parents
      @parents ||= []
    end

    def children
      @children ||= []
    end

    def parent
      @parents.first
    end

  end

  def final_path
    url = []

    m = self
    while m != nil
      unless m.key == '' or m.key.nil?
        url.unshift m.key
      end
      url.unshift m.class.path
      m = m.class.parent.nil? ? nil : m.send(m.class.parent)
    end

    url.join("/")
  end

  def save
    if self.key == '' or self.key.nil?
      self.set_variables InventosStreak.put(self.final_path, self.attributes)
    else
      self.set_variables InventosStreak.post(self.final_path, self.attributes)
    end
  end

  def destroy
    if self.key == '' or self.key.nil?
      raise ArgumentError.new("Cannot destroy a #{self.class.to_s} without a key")
    else
      InventosStreak.delete(self.final_path)
      self.key = nil
    end
  end

  def initialize args = {}
    set_parents
    set_children

    args.each do |k, v|
      if self.respond_to? k
        self.send("#{k}=", v)
      else
        raise ArgumentError.new("#{k} is not allowed.")
      end
    end
  end

  def set_variables hash
    hash.each do |k, v|
      self.instance_variable_set("@#{k}", v)
    end
  end

  def attributes
    h = {}
    self.instance_variables.each do |iv|
      val = self.instance_variable_get(iv)

      if val.class.to_s.include? 'InventosStreak::'
        h[iv.to_s.gsub('@', '').to_sym] = val.attributes
      else
        h[iv.to_s.gsub('@', '').to_sym] = val
      end
    end
    h
  end

  private

    def set_parents
      self.class.parents.each do |parent|
        singleton_class.class_eval do
          attr_reader parent
          define_method "#{parent}=" do |m|
            if m.class.to_s.downcase == "inventosstreak::#{parent.to_s}"
              instance_variable_set("@#{parent}", m)
            else
              raise ArgumentError.new("The model is not of the correct class.")
            end
          end
        end
      end
    end

    def set_children
      self.class.children.each do |child|
        singleton_class.class_eval do
          define_method "#{child}" do
            c = "InventosStreak::#{child.to_s.camelize.singularize}".constantize
            m = c.new
            m.send("#{self.class.to_s.split("::").last.underscore}=", self)

            puts m.final_path
            InventosStreak.get(m.final_path)
          end
        end
      end
    end
end